@@ -1,7 +1,7 @@
 
 /*
  *  autogen.h
- *  $Id: autogen.h,v 1.53 2004/02/15 02:36:09 bkorb Exp $
+ *  $Id: autogen.h,v 1.60 2004/10/11 23:30:00 bkorb Exp $
  *  Global header file for AutoGen
  */
 
@@ -27,8 +27,8 @@
 #ifndef AUTOGEN_HDR_H
 #define AUTOGEN_HDR_H
 
-#include "compat/compat.h"
 #include "config.h"
+#include "compat/compat.h"
 
 #if defined(HAVE_FOPENCOOKIE) || defined(HAVE_FUNOPEN)
 #  define ENABLE_FMEMOPEN
@@ -324,7 +324,7 @@ MODE tCC*        pzCurSfx         VALUE( NULL );
 MODE time_t      outTime          VALUE( 0 );
 MODE tFpStack*   pCurFp           VALUE( NULL );
 MODE tOutSpec*   pOutSpecList     VALUE( NULL );
-MODE jmp_buf     fileAbort        VALUE( { 0 } );
+MODE jmp_buf     fileAbort;
 MODE char*       pzCurStart       VALUE( NULL );
 MODE uintptr_t   curStartOff      VALUE( 0 );
 MODE tForInfo    forInfo          VALUE( { 0 } );
@@ -353,6 +353,13 @@ MODE tDefCtx     currDefCtx       VALUE( { NULL } );
 MODE tDefCtx     rootDefCtx       VALUE( { NULL } );
 MODE tTemplate*  pCurTemplate     VALUE( NULL );
 MODE tCC*        pzLastScheme     VALUE( NULL );
+#ifdef DAEMON_ENABLED
+/*
+ *  When operating as a daemon, autogen can be told to reload
+ *  its options the next time it wakes up (send it a SIGHUP).
+ */
+MODE ag_bool     redoOptions      VALUE( AG_TRUE );
+#endif
 
 /*
  *  Current Macro
@@ -399,16 +406,28 @@ MODE tDefEntry*  pCurrentEntry    VALUE( NULL );
 #define MKSTRING( name, val ) \
         MODE const char z ## name[ sizeof( val )] VALUE( val )
 
-MKSTRING( AllocWhat, "Could not allocate a %d byte %s\n" );
-MKSTRING( AllocErr,  "Allocation Failure" );
-MKSTRING( Cannot,    "fserr %d: cannot %s %s:  %s\n" );
-MKSTRING( TplWarn,   "Warning in template %s, line %d\n\t%s\n" );
-MKSTRING( FileLine,  "\tfrom %s line %d\n" );
-MKSTRING( ShDone,    "ShElL-OuTpUt-HaS-bEeN-cOmPlEtEd" );
-MKSTRING( NotStr,    "ERROR: %s is not a string\n" );
-MKSTRING( DevNull,   "/dev/null" );
-MKSTRING( ShellEnv,  "SHELL" );
-MKSTRING( Nil,       "" );
+MKSTRING( AllocWhat,  "Could not allocate a %d byte %s\n" );
+MKSTRING( AllocErr,   "Allocation Failure" );
+MKSTRING( Cannot,     "fserr %d: cannot %s %s:  %s\n" );
+MKSTRING( TplWarn,    "Warning in template %s, line %d\n\t%s\n" );
+MKSTRING( FileLine,   "\tfrom %s line %d\n" );
+MKSTRING( ShDone,     "ShElL-OuTpUt-HaS-bEeN-cOmPlEtEd" );
+MKSTRING( NotStr,     "ERROR: %s is not a string\n" );
+MKSTRING( DevNull,    "/dev/null" );
+MKSTRING( ShellEnv,   "SHELL" );
+
+/*
+ *  It may seem odd that there are two 'nil' strings.  It is used by
+ *  evalExpression() to distinguish between an actual value of a zero-
+ *  length string, and a defaulted value because there was no value
+ *  available.  (In fact, that routine will always select 'zDefaultNil'
+ *  except when "runShell()" returns NULL.  A NULL result means an empty
+ *  string that is valid (zNil) as opposed to an empty string due to
+ *  an unknown value (zDefaultNil).  This distinction is utilized in
+ *  the Select_Match_Existence() and Select_Match_NonExistence() functions.
+ */
+MKSTRING( Nil,        "" );
+MKSTRING( DefaultNil, "" );
 
 extern void unloadTemplate( tTemplate* pT );
 extern void unloadDefs( void );
@@ -449,6 +468,8 @@ MODE v2c_t p2p VALUE( { NULL } );
 #else
 # define AG_ABEND(s)  ag_abend(s)
 #endif
+#undef  DEBUG
+
 #define AG_ABEND_IN(t,m,s) \
     STMTS( pCurTemplate=(t); pCurMacro=(m); AG_ABEND(s);)
 
