@@ -1,6 +1,6 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
- * AutoGen Globally exported prototypes Tue May 18 20:49:08 PDT 2004
+ * AutoGen Globally exported prototypes Mon Oct 18 15:49:43 PDT 2004
  */
 #ifndef AUTOGEN_PROTOTYPES_H
 #define AUTOGEN_PROTOTYPES_H
@@ -10,6 +10,13 @@
 void
 loadCgi( void );
 
+/* exports from agInit.c */
+
+void
+initialize( int arg_ct, char** arg_vec );
+void
+handleSighup( int sig );
+
 /* exports from agShell.c */
 
 void
