@@ -1,6 +1,6 @@
 
 /*
- *  $Id: putshell.c,v 2.24 2004/01/01 20:06:01 bkorb Exp $
+ *  $Id: putshell.c,v 2.26 2004/07/22 02:46:22 bkorb Exp $
  *
  *  This module will interpret the options set in the tOptions
  *  structure and print them to standard out in a fashion that
@@ -8,7 +8,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2003 Bruce Korb
+ *  Automated Options copyright 1992-2004 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -175,7 +175,12 @@ putBourneShell( tOptions* pOpts )
                     (uintptr_t)(pOD->optCookie) );
             pOD->optCookie = (void*)(uintptr_t)~0UL;
             (*(pOD->pOptProc))( (tOptions*)2UL, pOD );
-            pz = (char*)pOD->pzLastArg;
+
+            /*
+             *  We are building the typeset list.  The list returned starts with
+             *  'none + ' for use by option saving stuff.  We must ignore that.
+             */
+            pz = (char*)pOD->pzLastArg + 7;
             while (*pz != NUL) {
                 printf( "typeset -x -i %s_", pOD->pz_NAME );
                 pz += strspn( pz, " +\t\n\f" );
